/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 18/10/22.
//

#ifndef FGGL_MESH_MESH_HPP
#define FGGL_MESH_MESH_HPP

#include "fggl/math/types.hpp"

#include <vector>
#include <span>

namespace fggl::mesh {

	struct Vertex3D {
		math::vec3 position;
		math::vec3 normal;
		math::vec3 colour{ 1.0F, 1.0F, 1.0f };
		math::vec2 texPos{ 0.0F, 0.0F };
	};

	struct Vertex2D {
		math::vec2 position;
		math::vec2 colour;
		math::vec2 texPos;
	};

	template<typename VertexFormat>
	struct Mesh {
		std::vector<VertexFormat> data;
		std::vector<uint32_t> indices;
	};

	template<typename MeshFormat>
	struct MultiMesh {
		std::vector<MeshFormat> meshes;
	};

	using Mesh2D = Mesh<Vertex2D>;
	using MultiMesh2D = MultiMesh<Mesh2D>;

	using Mesh3D = Mesh<Vertex3D>;
	using MultiMesh3D = MultiMesh<Mesh3D>;

}

#endif //FGGL_MESH_MESH_HPP
